<?php
class RSIS_Shipping_Model_Quote extends Mage_Shipping_Model_Carrier_Abstract
{
    /* Use group alias */
    protected $_code = 'rsisshipping';

    public function collectRates(Mage_Shipping_Model_Rate_Request $request)
    {
        // skip if not enabled
        if (!Mage::getStoreConfig('carriers/'.$this->_code.'/active'))
            return false;

        $result = Mage::getModel('shipping/rate_result');

        $endpoint = Mage::getStoreConfig('carriers/rsisshipping/endpoint');
        $apikey = Mage::getStoreConfig('carriers/rsisshipping/apikey');
        $endpoint = preg_replace("#/api.*$#","", $endpoint);
        $curl = curl_init($endpoint . '/api/shipment/quote/apikey');
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                "Authorization: RSIS $apikey"
        ));
        curl_setopt($curl, CURLOPT_FAILONERROR, true);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_TIMEOUT, 3);



        $packageDetails = array(
        	"weight"=>$request->getPackageWeight(),
        	"depth" => $request->getPackageDepth(),
        	"width" => $request->getPackageWidth(),
        	"height" => $request->getPackageHeight(),
        	"customsvalue"=>$request->getPackageValue()
        );
        if(empty($packageDetails['height'])) {
        	$packageParams = $request->getPackageParams();
        	if ($packageParams) {
        		$packageDetails['depth'] = max($packageDetails['depth'], $packageParams->getLength());
        		$packageDetails['width'] = max($packageDetails['width'], $packageParams->getWidth());
        		$packageDetails['height'] = max($packageDetails['height'], $packageParams->getHeight());
        	}
        	if(empty($packageDetails['height'])) { //Let's try to get it from the items instead
        		foreach($request->getAllItems() as $item){
	        		$product = $item->getProduct();
    	    		$packageDetails['depth'] = max($packageDetails['depth'], $product->getPackageDepth());
    	    		$packageDetails['depth'] = max($packageDetails['depth'], $product->getPackageLength());//alias of depth
        			$packageDetails['width'] = max($packageDetails['width'], $product->getPackageWidth());
        			$packageDetails['height'] = max($packageDetails['height'], $product->getPackageHeight());
        		}
        	}
        }
        $quote = array(
                "to" => array(
                        "zip"=> $request->getDestPostcode(),
                        "country"=>$request->getDestCountryId()
                ),
                "from" => array(
                        "zip"=>$request->getPostcode(),
                        "country" => $request->getCountryId()
                ),
                "packages" => array(
                     $packageDetails
                ),
        		"extras"=>array(
					"insurance"=>$request->getPackageValueWithDiscount()
                ),
                "contentType" => 'P',
                "shipmentDate" => date('Y-m-d'),
                "wt" => 'json',
        );
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($quote));
        $r = curl_exec($curl);
        $r = json_decode($r,true);

        if(is_array($r) && !empty($r['data']['quotes'])) {
	        foreach($r['data']['quotes'] as $index=>$quote) {
	        	if($quote['charges']['total'] <= 0) {
	        		continue; //Invalid setup, don't want to offer free shipping here.
	        	}
                $method = Mage::getModel('shipping/rate_result_method');
                $method->setCarrier($this->_code);
                /* Use method name */
                $method->setMethod('rsis_'.$quote['rateid']);
                $method->setMethodTitle($quote['name']);
                if(!empty($quote['charges']['originaltotal'])) {
                	$method->setCost($quote['charges']['originaltotal']);
                }
                $method->setPrice($quote['charges']['total']);
                $result->append($method);
	        }
        }

        return $result;
    }
}